@@ -1,7 +1,12 @@
 import base64
-import urllib2
 
-class PreemptiveBasicAuthHandler(urllib2.BaseHandler):
+try:
+    from urllib.request import BaseHandler, HTTPPasswordMgrWithDefaultRealm, build_opener
+except ImportError:
+    from urllib2 import BaseHandler, HTTPPasswordMgrWithDefaultRealm, build_opener
+
+
+class PreemptiveBasicAuthHandler(BaseHandler):
 
     def __init__(self, password_manager):
         self.password_manager = password_manager
@@ -13,7 +18,7 @@ class PreemptiveBasicAuthHandler(urllib2.BaseHandler):
             return request
 
         raw = "%s:%s" % (username, password)
-        auth = 'Basic %s' % base64.b64encode(raw).strip()
+        auth = 'Basic %s' % base64.b64encode(raw.encode('utf-8')).decode('utf-8').strip()
 
         request.add_unredirected_header('Authorization', auth)
         return request
@@ -21,8 +26,8 @@ class PreemptiveBasicAuthHandler(urllib2.BaseHandler):
 
 
 def basic_auth_opener(url, username, password):
-    password_manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
+    password_manager = HTTPPasswordMgrWithDefaultRealm()
     password_manager.add_password(None, url, username, password)
     auth_handler = PreemptiveBasicAuthHandler(password_manager)
-    opener = urllib2.build_opener(auth_handler)
+    opener = build_opener(auth_handler)
     return opener
