@@ -1,8 +1,11 @@
 import collections
 import re
-import urllib
-import urllib2
-import urlparse
+try:
+    from urllib.parse import quote
+    from urllib.request import HTTPError, Request
+except ImportError:
+    from urllib import quote
+    from urllib2 import HTTPError, Request
 
 from lxml import etree
 from lxml.builder import E
@@ -10,29 +13,17 @@ from lxml.builder import E
 from sharepoint.xml import SP, namespaces, OUT
 from sharepoint.lists import moderation
 from sharepoint.lists.types import type_mapping, default_type, UserField, LookupField
+from sharepoint.lists.attachments import SharePointAttachments
+from sharepoint.lists.definitions import LIST_WEBSERVICE, LIST_TEMPLATES
 from sharepoint.exceptions import UpdateFailedError
 
 uuid_re = re.compile(r'^\{?([\da-f]{8}-[\da-f]{4}-[\da-f]{4}-[\da-f]{4}-[\da-f]{12})\}?$')
 
-LIST_WEBSERVICE = '_vti_bin/Lists.asmx'
+try:
+    str = basestring
+except NameError:
+    pass
 
-# From http://msdn.microsoft.com/en-us/library/lists.lists.addlist%28v=office.12%29.aspx
-LIST_TEMPLATES = {
-    'Announcements': 104,
-    'Contacts': 105,
-    'Custom List': 100,
-    'Custom List in Datasheet View': 120,
-    'DataSources': 110,
-    'Discussion Board': 108,
-    'Document Library': 101,
-    'Events': 106,
-    'Form Library': 115,
-    'Issues': 1100,
-    'Links': 103,
-    'Picture Library': 109,
-    'Survey': 102,
-    'Tasks': 107
-}
 
 class SharePointLists(object):
     def __init__(self, opener):
@@ -99,7 +90,7 @@ class SharePointLists(object):
                 if list_object.id == key:
                     return list_object
             raise KeyError('No list with ID {0}'.format(key))
-        elif isinstance(key, basestring):
+        elif isinstance(key, str):
             for list_object in self.all_lists:
                 if list_object.meta['Title'] == key:
                     return list_object
@@ -164,7 +155,7 @@ class SharePointList(object):
         if not hasattr(self, '_rows'):
             attribs = collections.defaultdict(dict)
             field_groups, lookup_count = [[]], 0
-            for field in self.fields.itervalues():
+            for field in self.fields.values():
                 if isinstance(field, (UserField, LookupField)):
                     lookup_count += 1
                 if lookup_count >= 8:
@@ -183,7 +174,7 @@ class SharePointList(object):
                     attrib.update(row.attrib)
 
             self._rows = []
-            for attrib in attribs.itervalues():
+            for attrib in attribs.values():
                 self._rows.append(self.Row(attrib=attrib))
         return list(self._rows)
 
@@ -212,7 +203,7 @@ class SharePointList(object):
         """
         if not hasattr(self, '_row_class'):
             attrs = {'fields': self.fields, 'list': self, 'opener': self.opener}
-            for field in self.fields.itervalues():
+            for field in self.fields.values():
                 attrs[field.name] = field.descriptor
             self._row_class = type('SharePointListRow', (SharePointListRow,), attrs)
         return self._row_class
@@ -222,7 +213,7 @@ class SharePointList(object):
 
         if include_field_definitions:
             fields_element = OUT('fields')
-            for field in self.fields.itervalues():
+            for field in self.fields.values():
                 field_element = OUT('field',
                                   name=field.name,
                                   display_name=field.display_name,
@@ -254,8 +245,13 @@ class SharePointList(object):
             raise TypeError("row must be a dict or an instance of SharePointList.Row, not SharePointListRow")
         else:
             raise TypeError("row must be a dict or an instance of SharePointList.Row")
+        self.rows # Make sure self._rows exists.
         self._rows.append(row)
         return row
+    
+    def append_from(self, other_list):
+        for row in other_list.rows:
+            self.append(row.as_row(self.Row))
 
     def remove(self, row):
         """
@@ -346,7 +342,7 @@ class SharePointListRow(object):
         if isinstance(row, etree._Element):
             attrib = row.attrib
         if attrib:
-            for field in self.fields.itervalues():
+            for field in self.fields.values():
                 value = field.parse(attrib)
                 if value is not None:
                     self._data[field.name] = value
@@ -383,7 +379,7 @@ class SharePointListRow(object):
         batch_method = E.Method(Cmd='Update' if self.id else 'New')
         batch_method.append(E.Field(unicode(self.id) if self.id else 'New',
                                     Name='ID'))
-        for field in self.fields.itervalues():
+        for field in self.fields.values():
             if field.name in self._changed:
                 value = field.unparse(self._data[field.name] or '')
                 batch_method.append(E.Field(value, Name=field.name))
@@ -396,7 +392,7 @@ class SharePointListRow(object):
     def as_xml(self, transclude_xml=False, **kwargs):
         fields_element = OUT('fields')
         row_element = OUT('row', fields_element, id=unicode(self.id))
-        for field in self.fields.itervalues():
+        for field in self.fields.values():
             try:
                 data = self._data[field.name]
             except KeyError:
@@ -406,15 +402,38 @@ class SharePointListRow(object):
         if transclude_xml and self.is_file and self._data.get('DocIcon') == 'xml':
             try:
                 content = etree.parse(self.open()).getroot()
-            except urllib2.HTTPError, e:
+            except HTTPError as e:
                 content_element = OUT('content', missing='true')
             else:
                 content_element = OUT('content', content)
             row_element.append(content_element)
         return row_element
 
+    def as_dict(self, with_immutable=True, field_names=None):
+        data = {}
+        for field in self.fields.values():
+            if not with_immutable and field.immutable:
+                continue
+            if field_names is not None and field.name not in field_names:
+                continue
+            data[field.name] = getattr(self, field.name)
+        return data
+
+    def as_row(self, list_or_row):
+        row = list_or_row.Row if isinstance(list_or_row, SharePointList) else list_or_row
+        field_names = set(row.fields) & set(self.fields)
+        field_names -= set(['Attachments', '_Level', 'File_x0020_Type', '_CopySource', '_UIVersionString', 'FileLeafRef', 'Edit', 'LinkFilenameNoMenu', '_EditMenuTableEnd', '_ModerationComments', 'owshiddenversion', 'ContentType', 'ContentTypeId', '_HasCopyDestinations', 'EncodedAbsUrl', 'LinkTitle', 'WorkflowVersion', 'BaseName'])
+        field_names -= set(f for f in field_names if f.startswith('_'))
+        return row(self.as_dict(with_immutable=False, field_names=field_names))
+
     def open(self):
-        url = self.opener.relative(self.list.meta['Title'] + '/' + urllib.quote(self.LinkFilename.encode('utf-8')))
-        request = urllib2.Request(url)
+        url = self.opener.relative(self.list.meta['Title'] + '/' + quote(self.LinkFilename.encode('utf-8')))
+        request = Request(url)
         request.add_header('Translate', 'f')
         return self.opener.open(request)
+
+    @property
+    def attachments(self):
+        if not hasattr(self, '_attachments'):
+            self._attachments = SharePointAttachments(self.opener, self.list.id, self.id)
+        return self._attachments
